use ratatui::{
    layout::{
        Constraint,
        Direction,
        Layout,
        Rect
    },
    style::{Color, Style, Stylize},
    text::{Line, Span, Text},
    widgets::{Block, Borders, Clear, List, ListItem, Paragraph, Widget, Wrap},
    Frame,
};
use ratatui::crossterm::{event::{self, Event, KeyCode, KeyEvent, KeyEventKind}};

use crate::{client::Client, player::Player};

async fn handle_key_event(mut client: &mut Client, key_event: KeyEvent) {
    match key_event.code {
        KeyCode::Char('q') => Client::exit(client),
        KeyCode::Char('1') => {
            if !client.name.is_empty()  {
                println!("user already created");
                return
            }

            client.user = Some(Player::new().expect("toto"));
            client.name = client.user.clone().unwrap().name;
        },
        KeyCode::Char('2') => {
            todo!()
        },
        KeyCode::Char('3') => {
            if let Some(player) = client.user.clone() {
                let addr = client.addr.clone();
                Client::send_player(&mut client, &player, &addr).await.unwrap();
                return
            }
        }
        _ => {}
    }
}

pub async fn handle_events(client: &mut Client) -> anyhow::Result<()> {
    match event::read()? {
        Event::Key(key) if key.kind == KeyEventKind::Press => {
            handle_key_event(client, key).await
        }
        _ => {}
    };

    Ok(())
}

pub fn ui(frame: &mut Frame, app: &Client) {
    let chunks = Layout::default()
        .direction(Direction::Vertical)
        .constraints([
            Constraint::Min(3),
            Constraint::Min(1),
        ])
        .split(frame.area());

    let instructions = Line::from(vec![
        " 1".blue().bold(),
        " create player ".into(),
        "2".blue().bold(),
        " create lobby ".into(),
        "3".blue().bold(),
        " send player ".into(),
        "<Q>".blue().bold(),
        " quit ".into(),
    ]);

    let infos = Text::from(vec![
        Line::from(vec![
            "connected to: ".into(),
            app.addr.clone().yellow(),
        ]),
        Line::from(vec![
            "username: ".into(),
            app.name.clone().yellow(),
        ]),
    ]);

    let title = Line::from(" durak tui test fre ".bold());
    let top_block = Block::default()
        .title(title.centered())
        .borders(Borders::ALL)
        .style(Style::default());
    frame.render_widget(Paragraph::new(infos).block(top_block), chunks[0]);

    let server_message = Text::from(vec![
        Line::from(vec![
            "message: ".into(),
            app.response.clone().blue(),
        ])
    ]);

    let btm_block = Block::default()
        .title_bottom(instructions.centered())
        .borders(Borders::ALL)
        .style(Style::default());
    frame.render_widget(Paragraph::new(server_message).block(btm_block), chunks[1]);
}
